class NfeKeyError < StandardError
  attr_reader :key

  def initialize(key)
    @key =key
  end
end

module Nfe
  module Helpers

    class NfeKey

      attr_reader :uf, :cnpj, :series, :number, :code

      def initialize(uf, date, cnpj, series, number, code)
        @uf = uf
        @date = date
        @cnpj = cnpj
        @series = series
        @number = number
        @code = code
      end

      def self.parse(nfe_key)
        uf = Helpers::UfHelper.acronym(nfe_key[0..1].to_i)
        year   = nfe_key[2..3].to_i
        month  = nfe_key[4..5].to_i
        cnpj   = nfe_key[6..19]
        series = nfe_key[22..24].to_i
        number = nfe_key[25..33].to_i
        code   = nfe_key[35..42].to_i
        date = Date.new(year, month, 1)
        new(uf, date, cnpj, series, number, code)
      rescue
        raise NfeKeyError.new(nfe_key)
      end

      def key
        "#{key_without_check_digit}#{check_digit}"
      end

      def year
        @date.strftime("%y")
      end

      def month
        @date.strftime("%m")
      end

      def model
        55
      end

      def emission_kind
        1
      end

      def check_digit
        generate_check_digit(key_without_check_digit, weights)
      end

      private
      def key_without_check_digit
        "#{uf_code}#{year}#{month}#{cnpj}#{model}#{series_f}#{number_f}#{emission_kind}#{code_f}"
      end

      def uf_code
        Helpers::UfHelper.code(uf)
      end

      def series_f
        "%03d" % series
      end

      def number_f
        "%09d" % number
      end

      def code_f
        "%08d" % code
      end

      def weights
        "4329876543298765432987654329876543298765432".split("").map(&:to_i)
      end

      def generate_check_digit(values, weights)
        sum = reduce_weights(values, weights)
        mod = sum % 11
        mod < 2 ? 0 : (11 - mod)
      end

      def reduce_weights(values, weights)
        sum = 0
        weights.each_index do |i|
          sum += weights[i] * values[i].to_i
        end
        sum
      end
    end
  end
end
