module Nfe
  module Helpers
    module XmlModel
      class AttributeBase
        attr_accessor :name, :type, :format, :klass, :xml_alias

        def initialize(name, options={})
          @name = name

          @type = options[:type] || :text
          @required = options[:required]
          @format = options[:format]
          @klass = options[:klass]
          @xml_alias = (options[:xml_alias] || @name).to_s
        end

        def parse(context, doc)
          value = xml_value(doc)
          context.send("#{name}=", get_value(value)) if value.present?
        end

        def add_to_xml(context, xml, attribute)
          attribute = attr_accessor_value(context, attribute)

          if !attribute.present?
            xml.tag!(xml_alias) if required?
          else
            add_tag(xml, attribute)
          end
        end

        protected
        def get_value(value)
          value.text
        end

        def add_element(attribute)
          attribute.to_s
        end

        def xml_value(doc)
          doc.css(xml_alias).first
        end

        private
        def attr_accessor_value(context, attribute)
          context.send(attribute)
        end

        def add_tag(xml, attribute)
          xml.tag!(xml_alias) { |node| node << add_element(attribute) }
        end

        def required?
          @required
        end
      end
    end
  end
end
