module Nfe
  module Helpers
    module XmlModel
      module NfeModel
        def self.included(base)
          base.extend ClassMethods
        end

        module ClassMethods
          def attr_xml(xml_attr, options={})
            register_xml_attr xml_attr, options
            register_attr_acessor xml_attribute(xml_attr)
          end

          def xml_attributes
            @xml_attributes ||= {}
            if self.superclass.methods.include?(:xml_attributes)
              @xml_attributes = self.superclass.xml_attributes.merge @xml_attributes
            end
            @xml_attributes
          end

          def xml_attribute(attribute)
            xml_attributes[attribute]
          end

          private
          def register_xml_attr(xml_attr, options)
            xml_attributes[xml_attr] = Factory.create xml_attr, options
          end

          def register_attr_acessor(xml_attr)
            instance_eval do
              attr_accessor xml_attr.name unless xml_attr.type.equal? :enumerize
            end
          end
        end

        def present?
          check_present
        end

        def check_present
          present = false
          self.class.xml_attributes.each do |key, value|
            present ||= value_present? key, value
          end
          present
        end

        def parse!(xml)
          parse_elements xml
        end

        def to_xml
          xml = Builder::XmlMarkup.new

          add_elements xml

          xml.target!
        end

        def parse_elements(doc, *elements)
          doc = Nokogiri::XML(doc) if doc.is_a? String

          elements = get_elements(elements)

          Array(elements).each { |attribute| parse_value doc, attribute }
        end

        def add_elements(xml, *elements)
          elements = get_elements(elements)

          Array(elements).each { |attribute| add_element xml, attribute }
        end

        def validate_optional_attributes(*attributes)
          attributes.each do |attribute|
            Array(send(attribute)).each do |item|
              if item.present? && !item.valid?
                errors.add(attribute, :invalid) unless errors.include?(attribute)
              end
            end
          end
        end

        def validate_required_attributes(*attributes)
          attributes.each do |attribute|
            if !send(attribute).valid?
              errors.add(attribute, :invalid) unless errors.include?(attribute)
            end
          end
        end

        private
        def parse_value(doc, attribute)
          self.class.xml_attribute(attribute).parse self, doc
        end

        def add_element(xml, attribute)
          self.class.xml_attribute(attribute).add_to_xml self, xml, attribute
        end

        def get_elements(elements)
          elements = self.class.xml_attributes.keys if elements.empty?

          elements
        end

        def value_present?(key, value)
          if value.type.equal? :bigdecimal
            send(key) > 0
          else
            send(key).present?
          end
        end
      end
    end
  end
end
