module Nfe
  module Models
    class NFe
      include ActiveModel::Model
      include Helpers::XmlModel::NfeModel

      attr_accessor :infNFe, :signature

      validate { validate_required_attributes :infNFe }

      def initialize
        @infNFe = Nfe::Models::InfNFe.new
        @signature = Nfe::Models::Signature.new
      end

      def self.namespace
        "http://www.portalfiscal.inf.br/nfe"
      end

      def parse!(xml)
        doc = Nokogiri::XML(xml)

        @infNFe.parse!(doc.css("infNFe").to_xml)

        sign = doc.xpath("//ds:Signature", ds: Signature.namespace).to_xml
        @signature.parse!(sign)
      end

      def to_xml
        xml = Builder::XmlMarkup.new

        xml.NFe(xmlns: Nfe::Models::NFe.namespace) do |nfe|

          nfe.infNFe({versao: infNFe.versao, Id: infNFe.id}) do |node|
            node << @infNFe.to_xml
          end

          nfe.Signature(xmlns: Signature.namespace) do |node|
            node << @signature.value
          end
        end

        xml.target!
      end
    end
  end
end
