module Nfe
  module Models
    class Avulsa
      include ActiveModel::Model
      include Helpers::XmlModel::NfeModel

      attr_xml :cnpj, required: true, xml_alias: "CNPJ"
      attr_xml :xOrgao, required: true
      attr_xml :matr, required: true
      attr_xml :xAgente, required: true
      attr_xml :fone 
      attr_xml :uf, required: true, xml_alias: "UF"
      attr_xml :nDAR
      attr_xml :dEmi, type: :date
      attr_xml :vDAR, type: :float
      attr_xml :repEmi, required: true
      attr_xml :dPag, type: :date

      validates :cnpj,
        presence: true,
        cnpj: true,
        length: { is: 14}

      validates :xOrgao,
        length: { in: 1..60 }

      validates :matr,
        length: { in: 1..60 }

      validates :xAgente,
        length: { in: 1..60 }

      validates :fone,
        length: { in: 6..14, allow_blank: true },
        numericality: { only_integer: true, allow_blank: true }

      validates :uf,
        length: { is: 2 }

      validates :nDAR,
        length: { in: 1..60, allow_blank: true }

      validates :dEmi,
        length: { is: 10, allow_blank: true }

      validates :vDAR,
        numericality: {
          greater_than_or_equal_to: 0,
          less_than_or_equal_to: 9_999_999_999_999.99,
          allow_blank: true }

      validates :repEmi,
        length: { in: 1..60 }

      validates :dPag,
        length: { is: 10, allow_blank: true }
    end
  end
end
