require_relative "bigdecimal_less_than_validator"
require_relative "bigdecimal_greater_than_validator"

module Nfe
  module Models
    module Cane
      class Cana
        include ActiveModel::Model
        include Helpers::XmlModel::NfeModel

        attr_xml :safra, required: true
        attr_xml :ref, required: true
        attr_xml :vFor, type: :float, required: true
        attr_xml :vTotDed, type: :float, required: true
        attr_xml :vLiqFor, type: :float, required: true
        attr_xml :forDia, type: :collection, klass: "Nfe::Models::Cane::ForDia"
        attr_xml :deduc, type: :collection, klass: "Nfe::Models::Cane::Deduc"
        attr_xml :qTotMes, type: :bigdecimal, required: true
        attr_xml :qTotAnt, type: :bigdecimal, required: true
        attr_xml :qTotGer, type: :bigdecimal, required: true

        validates :safra,
          length: { in: 4..9 }

        validates :ref,
          length: { is: 7 }

        validates :qTotMes,
          bigdecimal_less_than: true,
          bigdecimal_greater_than: true

        validates :qTotAnt,
          bigdecimal_less_than: true,
          bigdecimal_greater_than: true

        validates :qTotGer,
          bigdecimal_less_than: true,
          bigdecimal_greater_than: true

        validates :vFor,
          presence: true,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99
          }

        validates :vTotDed,
          presence: true,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to:  9_999_999_999_999.99
          }

        validates :vLiqFor,
          presence: true,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to:  9_999_999_999_999.99
          }

        validate { validate_optional_attributes :forDia, :deduc }

        def initialize
          @qTotMes = BigDecimal("0", 22)
          @qTotAnt = BigDecimal("0", 22)
          @qTotGer = BigDecimal("0", 22)
          @forDia = []
          @deduc = []
        end

        def qTotMes=(value)
          @qTotMes = BigDecimal(value, 22)
        end

        def qTotAnt=(value)
          @qTotAnt = BigDecimal(value, 22)
        end

        def qTotGer=(value)
          @qTotGer = BigDecimal(value, 22)
        end

        def to_xml
          xml = Builder::XmlMarkup.new

          add_elements xml, :safra, :ref

          @forDia.each do |forDia|
            xml.forDia(dia: forDia.dia) { |node| node << forDia.to_xml }
          end

          add_elements xml, :qTotMes, :qTotAnt, :qTotGer, :deduc, :vFor,
            :vTotDed, :vLiqFor

          xml.target!
        end
      end
    end
  end
end
