module Nfe
  module Models
    module Emitter
      class Emit
        include ActiveModel::Model
        include Helpers::XmlModel::NfeModel
        include Concerns::CnpjCpfValidatable

        attr_xml :cnpj, xml_alias: "CNPJ"
        attr_xml :cpf, xml_alias: "CPF"
        attr_xml :xNome, required: true
        attr_xml :xFant
        attr_xml :ender, type: :node, required: true, xml_alias: "enderEmit"
        attr_xml :ie, xml_alias: "IE", required: true
        attr_xml :ieST, xml_alias: "IEST"
        attr_xml :im, xml_alias: "IM"
        attr_xml :cnae, xml_alias: "CNAE"
        attr_xml :crt, type: :enumerize, required: true, xml_alias: "CRT"

        extend Enumerize

        enumerize :crt, in: { simple: 1, exceeded_simple: 2, normal: 3 }

        validates :xNome,
          presence: true,
          length: { in: 2..60 }

        validates :xFant,
          length: { maximum: 60, allow_blank: true }

        validates :ie,
          presence: true,
          ie: { uf: "ender#uf" },
          length: { in: 2..14 }

        validates :ieST,
          length: { in: 2..14, allow_blank: true }

        validates :im,
          length: { in: 2..15, allow_blank: true }

        validates :cnae,
          length: { maximum: 7, allow_blank: true }

        validates :crt,
          presence: true

        validate { validate_required_attributes :ender }

        def initialize
          @ender = Ender.new
        end
      end
    end
  end
end
