module Nfe
  module Models
    class Ender
      include ActiveModel::Model
      include Helpers::XmlModel::NfeModel

      attr_xml :xLgr, required: true
      attr_xml :nro, required: true
      attr_xml :xCpl
      attr_xml :xBairro, required: true
      attr_xml :cMun, required: true, format: "%07d"
      attr_xml :xMun, required: true
      attr_xml :uf, required: true, xml_alias: "UF"
      attr_xml :cep, required: true, xml_alias: "CEP"
      attr_xml :cPais
      attr_xml :xPais
      attr_xml :fone

      validates :xLgr,
        presence: true,
        length: { in: 2..60 }

      validates :nro,
        presence: true,
        length: { maximum: 60 }

      validates :xCpl,
        length: { maximum: 60 }

      validates :xBairro,
        presence: true,
        length: { in: 2..60 }

      validates :cMun,
        presence: true,
        length: { maximum: 7 },
        numericality: { only_integer: true }

      validates :xMun,
        presence: true,
        length: { in: 2..60 }

      validates :uf,
        presence: true,
        length: { is: 2 }

      validates :cep,
        presence: true,
        length: { is: 8 },
        numericality: { only_integer: true }

      validates :cPais,
        length: { is: 4, allow_blank: true },
        numericality: { only_integer: true, allow_blank: true }

      validates :xPais,
        length: { maximum: 60 }

      validates :fone,
        length: { in: 6..14, allow_blank: true },
        numericality: { only_integer: true, allow_blank: true }
    end
  end
end
