module Nfe
  module Models
    class Evento
      include ActiveModel::Model
      include Helpers::XmlModel::NfeModel

      attr_xml :tpAmb, type: :enumerize, required: true
      attr_xml :cnpj, xml_alias: "CNPJ"
      attr_xml :cpf, xml_alias: "CPF"
      attr_xml :chNFe, required: true
      attr_xml :dhEvento, type: :datetime, required: true
      attr_xml :tpEvento, required: true
      attr_xml :nSeqEvento, required: true
      attr_xml :versao, required: true, xml_alias: "verEvento"

      attr_accessor :uf, :detEvento, :signature

      extend Enumerize

      enumerize :tpAmb, in: { production: 1, test: 2 }

      validates :versao,
        length: { is: 4 }

      validates :uf,
        presence: true,
        length: { is: 2 }

      validates :tpEvento,
        length: { is: 6 }

      validates :nSeqEvento,
        length: { in: 1..2 }

      validates :chNFe,
        length: { is: 44 }

      validate { validate_required_attributes :detEvento }

      def id
        "ID#{tpEvento}#{chNFe}#{nSeqEvento_f}"
      end

      def to_xml
        xml = Builder::XmlMarkup.new

        xml.evento({xmlns: Nfe::Models::NFe.namespace, versao: versao}) do |evento|

          evento.infEvento(Id: id) do |infEvento|
            infEvento.cOrgao Nfe::Helpers::UfHelper.code(uf)

            add_elements infEvento

            infEvento << detEvento.to_xml
          end

          evento.Signature(xmlns: Models::Signature.namespace) do |node|
            node << @signature.value
          end

        end
        xml.target!
      end

      private
      def nSeqEvento_f
        "%02d" % nSeqEvento
      end
    end
  end
end
