module Nfe
  module Models
    class Ide
      include ActiveModel::Model
      include Concerns::Ufable
      include Helpers::XmlModel::NfeModel

      attr_xml :cUF, type: :integer, required: true
      attr_xml :cNF, required: true, format: "%08d"
      attr_xml :natOp, required: true
      attr_xml :serie, required: true
      attr_xml :indPag, type: :enumerize, required: true
      attr_xml :tpNF, type: :enumerize, required: true
      attr_xml :idDest, type: :enumerize, required: true
      attr_xml :tpImp, type: :enumerize, required: true
      attr_xml :tpEmis, type: :enumerize, required: true
      attr_xml :tpAmb, type: :enumerize, required: true
      attr_xml :finNFe, type: :enumerize, required: true
      attr_xml :nNF, required: true
      attr_xml :dhEmi, type: :datetime, required: true
      attr_xml :dhSaiEnt, type: :datetime
      attr_xml :cMunFG, required: true, format: "%07d"
      attr_xml :NFref, type: :collection, klass: "Nfe::Models::Referenced::NFref"
      attr_xml :cDV, required: true
      attr_xml :verProc, required: true
      attr_xml :dhCont, type: :datetime
      attr_xml :xJust, type: :text

      extend Enumerize

      enumerize :indPag, in: { cash_payment: 0, installment_payment: 1, others: 2 }

      enumerize :idDest, in: { state: 1, interstate: 2, external: 3 }

      enumerize :tpNF, in: { entry: 0, departure: 1 }

      enumerize :tpImp, in: { portrait: 1, landscape: 2 }

      enumerize :tpEmis, in: { normal: 1, fs: 2, scan: 3, dpec: 4, fs_da: 5,
        svc_an: 6, svc_rs: 7 }

      enumerize :tpAmb, in: { production: 1, test: 2 }

      enumerize :finNFe, in: { normal: 1, complement: 2, adjustment: 3, returning: 4 }

      validates :cUF,
        presence: true,
        length: { is: 2 }

      validates :cNF,
        presence: true,
        numericality: true,
        length: { maximum: 8 }

      validates :natOp,
        presence: true,
        length: { maximum: 60 }

      validates :indPag,
        presence: true

      validates :serie,
        presence: true,
        numericality: true,
        length: { maximum: 3 }

      validates :idDest,
        presence: true

      validates :nNF,
        presence: true,
        numericality: true,
        length: { in: 1..9 }

      validates :dhEmi,
        presence: true

      validates :tpNF,
        presence: true

      validates :cMunFG,
        presence: true,
        length: { maximum: 7 }

      validates :tpImp,
        presence: true

      validates :tpEmis,
        presence: true

      validates :cDV,
        presence: true,
        numericality: true,
        length: { is: 1 }

      validates :tpAmb,
        presence: true

      validates :finNFe,
        presence: true

      validates :verProc,
        presence: true,
        length: { maximum: 20 }

      validate { validate_optional_attributes :NFref }

      def initialize
        @NFref = []
      end

      def to_xml
        xml = Builder::XmlMarkup.new

        add_elements xml, :cUF, :cNF, :natOp, :indPag

        xml.mod MODEL

        add_elements xml, :serie, :nNF, :dhEmi, :dhSaiEnt, :tpNF, :idDest,
          :cMunFG, :tpImp, :tpEmis, :cDV, :tpAmb, :finNFe

        xml.indFinal NOT_CONSUMER

        xml.indPres NOT_PRESENT

        xml.procEmi APP_TAX_PAYER

        add_elements xml, :verProc, :dhCont, :xJust, :NFref

        xml.target!
      end

      private
      APP_TAX_PAYER = 0
      NOT_CONSUMER = 0
      MODEL = 55
      NOT_PRESENT = 9
    end
  end
end
