module Nfe
  module Models
    class InfNFe
      include ActiveModel::Model
      include Helpers::XmlModel::NfeModel

      attr_xml :versao, type: :attribute
      attr_xml :ide, type: :node, required: true
      attr_xml :emit, type: :node, required:true
      attr_xml :avulsa, type: :node
      attr_xml :dest, type: :node, required:true
      attr_xml :retirada, type: :node
      attr_xml :entrega, type: :node
      attr_xml :det, type: :collection, klass: "Nfe::Models::Det"
      attr_xml :total, type: :node, required:true
      attr_xml :transp, type: :node, required:true
      attr_xml :cobr, type: :node
      attr_xml :infAdic, type: :node
      attr_xml :exporta, type: :node
      attr_xml :compra, type: :node
      attr_xml :cana, type: :node

      attr_accessor :signature

      validates :versao,
        length: { in: 1..4 }

      validate do
        validate_required_attributes :ide, :emit, :dest, :total, :transp

        validate_optional_attributes :avulsa, :retirada, :entrega, :cobr,
          :infAdic, :exporta, :compra, :cana, :det
      end

      def initialize
        @ide = Ide.new
        @emit = Emitter::Emit.new
        @avulsa = Avulsa.new
        @dest = Recipient::Dest.new
        @retirada = EntregaRetirada.new
        @entrega = EntregaRetirada.new
        @det = []
        @total = Total.new
        @transp = Transport::Transp.new
        @cobr = Bill::Cobr.new
        @infAdic = Information::InfAdic.new
        @exporta = Exporta.new
        @compra = Compra.new
        @cana = Cane::Cana.new
      end

      def id
        "NFe#{nfe_key.key}"
      end

      def key
        nfe_key.key
      end

      def to_xml
        xml = Builder::XmlMarkup.new

        add_elements xml, :ide, :emit, :avulsa, :dest, :retirada, :entrega

        @det.each do |det|
          xml.det(nItem: det.nItem) { |node| node << det.to_xml }
        end

        add_elements xml, :total, :transp, :cobr, :infAdic, :exporta, :compra,
          :cana

        xml.target!
      end

      private
      def nfe_key
        Helpers::NfeKey.new(ide.uf, ide.dhEmi, emit.cnpj, ide.serie, ide.nNF,
          ide.cNF)
      end
    end
  end
end
