module Nfe
  module Models
    module Information
      class InfAdic
        include ActiveModel::Model
        include Helpers::XmlModel::NfeModel

        attr_xml :infAdFisco
        attr_xml :infCpl
        attr_xml :obsCont, type: :collection, klass: "Nfe::Models::Information::ObsContFisco"
        attr_xml :obsFisco, type: :collection, klass: "Nfe::Models::Information::ObsContFisco"
        attr_xml :procRef, type: :collection, klass: "Nfe::Models::Information::ProcRef"

        validates :infAdFisco,
          length: { maximum: 2000 }

        validates :infCpl,
          length: { maximum: 5000 }

        validates :obsCont, :obsFisco,
           length: { maximum: 10, message: :less_than_or_equal_to }

        validate { validate_optional_attributes :obsFisco, :obsCont, :procRef }

        def initialize
          @obsFisco = []
          @obsCont = []
          @procRef = []
        end

        def to_xml
          xml = Builder::XmlMarkup.new

          add_elements xml, :infAdFisco, :infCpl

          @obsCont.each do |obsCont|
            xml.obsCont(xCampo: obsCont.xCampo) { |node| node << obsCont.to_xml }
          end

          @obsFisco.each do |obsFisco|
            xml.obsFisco(xCampo: obsFisco.xCampo) { |node| node << obsFisco.to_xml }
          end

          add_elements xml, :procRef

          xml.target!
        end
      end
    end
  end
end
