module Nfe
  module Models
    module Itens
      class Comb
        include ActiveModel::Model
        include Helpers::XmlModel::NfeModel

        attr_xml :cProdANP, required: true
        attr_xml :codif, xml_alias: "CODIF"
        attr_xml :qTemp, type: :float, format: "%0.4f"
        attr_xml :ufCons, required: true, xml_alias: "UFCons"
        attr_xml :cide, type: :node, xml_alias: "CIDE"

        validates :cProdANP,
          length: { is: 9 },
          numericality: { only_integer: true }

        validates :codif,
          length: { maximum: 21 },
          numericality: { only_integer: true, allow_blank: true }

        validates :qTemp,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 999_999_999_999.9999,
            allow_blank: true
          }

        validates :ufCons,
          presence: true,
          length: { is: 2 }

        validate { validate_optional_attributes :cide }  

        def initialize
          @cide = Nfe::Models::Itens::CIDE.new
        end
      end
    end
  end
end
