module Nfe
  module Models
    module Itens
      class Prod
        include ActiveModel::Model
        include Helpers::XmlModel::NfeModel

        attr_xml :cProd, required: true
        attr_xml :cEAN, required: true
        attr_xml :xProd, required: true
        attr_xml :ncm, required: true, xml_alias: "NCM"
        attr_xml :extipi, xml_alias: "EXTIPI"
        attr_xml :cfop, required: true, xml_alias: "CFOP"
        attr_xml :uCom, required: true
        attr_xml :qCom, required: true
        attr_xml :vUnCom, required: true
        attr_xml :vProd, required: true, type: :float
        attr_xml :cEANTrib, required: true
        attr_xml :uTrib, required: true
        attr_xml :qTrib, required: true
        attr_xml :vUnTrib, required: true
        attr_xml :vFrete, type: :float
        attr_xml :vSeg, type: :float
        attr_xml :vDesc, type: :float
        attr_xml :vOutro, type: :float
        attr_xml :indTot, required: true
        attr_xml :di, type: :collection, xml_alias: "DI",
          klass: "Nfe::Models::Itens::DI"
        attr_xml :xPed
        attr_xml :nItemPed
        attr_xml :veicProd, type: :node
        attr_xml :med, type: :collection, klass: "Nfe::Models::Itens::Med"
        attr_xml :arma, type: :collection, klass: "Nfe::Models::Itens::Arma"
        attr_xml :comb, type: :node

        def initialize
          @di = []
          @veicProd = VeicProd.new
          @med = []
          @arma = []
          @comb = Comb.new
        end

        validate do
          validate_optional_attributes :di, :veicProd, :med, :arma, :comb
        end

        validate :cean_valid_lenght, :ncm_valid_lenght

        validates :cProd,
          length: { in: 1..60 }

        validates :xProd,
          length: { in: 1..120 }

        validates :extipi,
          length: { in: 2..3, allow_blank: true}

        validates :cfop,
          length: { is: 4}

        validates :uCom, :uTrib,
          length: { in: 1..6 }

        validates :vUnCom, :vUnTrib,
          presence: true,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999_999_999.99
          }

        validates :qCom, :vProd, :qTrib,
          presence: true,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99
          }

        validates :vFrete, :vSeg, :vDesc, :vOutro,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99,
            allow_blank: true
          }

        validates :indTot,
          length: { is: 1 }

        validates :xPed,
          length: { in: 1..15, allow_blank: true }

        validates :nItemPed,
          length: { is: 6, allow_blank: true }

        private
        def ncm_valid_lenght
          valid_lenght :ncm, [0, 2, 8], @ncm
        end

        def cean_valid_lenght
          [:cEAN, :cEANTrib].each do |attribute|
            valid_lenght attribute, [0, 8, 12, 13, 14], send(attribute)
          end
        end

        def valid_lenght(attribute_name, valid_values, value)
          unless valid_values.include? value.to_s.length
            errors.add(attribute_name, I18n.t("errors.messages.wrong_length",
              count: valid_values.to_s))
          end
        end
      end
    end
  end
end
