module Nfe
  module Models
    module Itens
      class VeicProd
        include ActiveModel::Model
        include Helpers::XmlModel::NfeModel

        attr_xml :tpOp, required: true
        attr_xml :chassi, required: true
        attr_xml :cCor, required: true
        attr_xml :xCor, required: true
        attr_xml :pot, required: true
        attr_xml :cilin, required: true
        attr_xml :pesoL, required: true
        attr_xml :pesoB, required: true
        attr_xml :nSerie, required: true
        attr_xml :tpComb, required: true
        attr_xml :nMotor, required: true
        attr_xml :cmt, required: true, xml_alias: "CMT"
        attr_xml :dist, required: true
        attr_xml :anoMod, required: true
        attr_xml :anoFab, required: true
        attr_xml :tpPint, required: true
        attr_xml :tpVeic, required: true
        attr_xml :espVeic, required: true
        attr_xml :vin, required: true, xml_alias: "VIN"
        attr_xml :condVeic, required: true
        attr_xml :cMod, required: true
        attr_xml :cCorDENATRAN, required: true
        attr_xml :lota, required: true
        attr_xml :tpRest, required: true

        validates :tpOp,
          presence: true,
          inclusion: { in: ["0", "1", "2", "3"] }

        validates :chassi,
          presence: true,
          length: { is: 17 }

        validates :cCor,
          presence: true,
          length: { maximum: 4 }

        validates :xCor,
          presence: true,
          length: { maximum: 40 }

        validates :pot,
          presence: true,
          length: { maximum: 4 }

        validates :cilin,
          presence: true,
          length: { is: 4 }

        validates :pesoL,
          presence: true,
          length: { maximum: 9 }

        validates :pesoB,
          presence: true,
          length: { maximum: 9 }

        validates :nSerie,
          presence: true,
          length: { maximum: 9 }

        validates :tpComb,
          presence: true,
          length: { maximum: 2 },
          numericality: { only_integer: true }

        validates :nMotor,
          presence: true,
          length: { maximum: 21 }

        validates :cmt,
          presence: true,
          length: { maximum: 9 }

        validates :dist,
          presence: true,
          length: { maximum: 4 },
          numericality: { only_integer: true }

        validates :anoMod,
          presence: true,
          length: { is: 4 },
          numericality: { only_integer: true }

        validates :anoFab,
          presence: true,
          length: { is: 4 },
          numericality: { only_integer: true }

        validates :tpPint,
          presence: true,
          length: { is: 1 }

        validates :tpVeic,
          presence: true,
          length: { maximum: 2 },
          numericality: { only_integer: true }

        validates :espVeic,
          presence: true,
          length: { is: 1 },
          numericality: { only_integer: true }

        validates :vin,
          presence: true,
          inclusion: { in: ["R", "N"] }

        validates :condVeic,
          presence: true,
          inclusion: { in: ["1", "2", "3"] }

        validates :cMod,
          presence: true,
          length: { maximum: 6 },
          numericality: { only_integer: true }

        validates :cCorDENATRAN,
          presence: true,
          length: { maximum: 2 },
          numericality: { only_integer: true }

        validates :lota,
          presence: true,
          length: { maximum: 3 },
          numericality: { only_integer: true }

        validates :tpRest,
          presence: true,
          inclusion: { in: ["0", "1", "2", "3", "4", "9"] }
      end
    end
  end
end
