module Nfe
  module Models
    module Recipient
      class Dest
        include ActiveModel::Model
        include Helpers::XmlModel::NfeModel

        attr_xml :cnpj, xml_alias: "CNPJ"
        attr_xml :cpf, xml_alias: "CPF"
        attr_xml :xNome, required: true
        attr_xml :ender, type: :node, required: true, xml_alias: "enderDest"
        attr_xml :indIEDest, type: :enumerize, required: true
        attr_xml :ie, xml_alias: "IE"
        attr_xml :isuf, xml_alias: "ISUF"
        attr_xml :email

        extend Enumerize

        enumerize :indIEDest, in: { taxpayer: 1, immune: 2, not_taxpayer: 9 }

        validates :cnpj,
          presence: {if: -> { @cpf.blank? and @ender.uf != "EX" } },
          cnpj: true,
          length: { is: 14, allow_blank: true }

        validates :cpf,
          presence: {if: -> { @cnpj.blank? and @ender.uf != "EX" } },
          cpf: true,
          length: { is: 11, allow_blank: true }

        validates :xNome,
          length: { in: 2..60 }

        validates :ie,
          ie: { uf: "ender#uf" },
          length: { in: 2..14, allow_blank: true }

        validates :isuf,
          length: { in: 8..9, allow_blank: true }

        validates :indIEDest,
          presence: true

        validates :email,
          length: { maximum: 60, allow_blank: true },
          format: { with: /\A([^@\s]+)@((?:[-a-z0-9]+\.)+[a-z]{2,})\z/i,
            allow_blank: true }

        validate { validate_optional_attributes :ender }

        def initialize
          @ender = Ender.new
        end
      end
    end
  end
end
