module Nfe
  module Models
    module Referenced
      class NFref
        include ActiveModel::Model
        include Helpers::XmlModel::NfeModel

        attr_xml :refNFe
        attr_xml :refNF, type: :node
        attr_xml :refNFP, type: :node
        attr_xml :refCTe
        attr_xml :refECF, type: :node

        validate :need_at_least_one_attribute_present

        validates :refNFe,
          length: { is: 44, allow_blank: true }

        validates :refCTe,
          length: { is: 44, allow_blank: true }

        validate { validate_optional_attributes :refNF, :refNFP, :refECF }

        def initialize
          @refNF = Nfe::Models::Referenced::RefNF.new
          @refNFP = Nfe::Models::Referenced::RefNFP.new
          @refECF = Nfe::Models::Referenced::RefECF.new
        end

        def present?
          true
        end

        private
        def need_at_least_one_attribute_present
          if !@refNFe.present? && !@refNF.present? && !@refNFP.present?  &&
            !@refCTe.present? && !@refECF.present?
            errors.add(:NFref, I18n.t("errors.messages.empty"))
          end
        end
      end
    end
  end
end
