module Nfe
  module Models
    class Signature
      include ActiveModel::Model

      attr_accessor :id

      attr_writer :value

      validates :id,
        presence: true

      def initialize(id = nil)
        @id = id
      end

      def self.namespace
        "http://www.w3.org/2000/09/xmldsig#"
      end

      def value
        @value || unsigned
      end

      def parse!(xml)
        doc = Nokogiri::XML(xml)
        @value = doc.children.present? ? doc.root.children.to_xml : nil
      end

      private
      def unsigned
        xml  = "<SignedInfo>"
        xml +=   "<CanonicalizationMethod Algorithm='http://www.w3.org/TR/2001/REC-xml-c14n-20010315'/>"
        xml +=   "<SignatureMethod Algorithm='http://www.w3.org/2000/09/xmldsig#rsa-sha1'/>"
        xml +=   "<Reference URI='##{@id}'>"
        xml +=     "<Transforms>"
        xml +=       "<Transform Algorithm='http://www.w3.org/2000/09/xmldsig#enveloped-signature'/>"
        xml +=       "<Transform Algorithm='http://www.w3.org/TR/2001/REC-xml-c14n-20010315'/>"
        xml +=     "</Transforms>"
        xml +=     "<DigestMethod Algorithm='http://www.w3.org/2000/09/xmldsig#sha1'/>"
        xml +=     "<DigestValue></DigestValue>"
        xml +=   "</Reference>"
        xml += "</SignedInfo>"
        xml += "<SignatureValue></SignatureValue>"
        xml += "<KeyInfo>"
        xml +=   "<X509Data>"
        xml +=     "<X509Certificate></X509Certificate>"
        xml +=   "</X509Data>"
        xml += "</KeyInfo>"
      end
    end
  end
end
