module Nfe
  module Models
    module Taxes
      class ICMS
        include ActiveModel::Model

        def self.attr_icms
          [:orig, :csosn, :cst, :modBC, :vBC, :pRedBC, :pICMS, :vICMS, :modBCST,
            :pMVAST, :pRedBCST, :vBCST, :pICMSST, :vICMSST, :pCredSN,
            :vCredICMSSN, :motDesICMS, :vBCSTRet, :vICMSSTRet, :vICMSDeson]
        end

        def self.cst_types
          ["00", "10", "20", "30", "40", "41", "50", "51", "60",
            "70", "90", "101", "102", "103", "300", "400", "201", "202", "203", "500",
            "900"]
        end

        attr_accessor :orig, :cst, :modBC, :vBC, :pRedBC, :pICMS, :vICMS, :modBCST,
          :pMVAST, :pRedBCST, :vBCST, :pICMSST, :vICMSST, :pCredSN,
          :vCredICMSSN, :motDesICMS, :vBCSTRet, :vICMSSTRet, :vICMSDeson

        alias_attribute :csosn, :cst

        validates :cst,
          presence: true,
          inclusion: { in: cst_types }

        def present?
          [@orig, @cst, @modBC, @vBC, @pRedBC, @pICMS, @vICMS, @modBCST,
            @pMVAST, @pRedBCST, @vBCST, @pICMSST, @vICMSST, @pCredSN,
            @vCredICMSSN, @motDesICMS, @vBCSTRet, @vICMSSTRet].
              reduce(false) {|memo, obj| memo || obj.present?}
        end

        def parse!(xml)
          doc = Nokogiri::XML(xml)
          reset_values
          parse_xml(doc)
        end

        def to_xml
          xml = Builder::XmlMarkup.new

          to_object_xml(xml)

          xml.target!
        end

        private
        def to_object_xml(xml)
          icms_type = Taxes::const_get("ICMS#{@cst}")

          if icms_type

            icms = icms_type.new

            attribute_to_object icms

            xml.tag!(icms_type.root_name) { |node| node << icms.to_xml }
          end
        end

        def attribute_to_object(object)
          self.class.attr_icms.each do |attribute|
            if object.respond_to? "#{attribute}="
              object.send("#{attribute}=", send(attribute))
            end
          end
        end

        def parse_xml(doc)
          self.class.cst_types.each do |cst|
            icms_type = Taxes::const_get("ICMS#{cst}")

            if doc.css(icms_type.root_name).present?
              icms = icms_type.new

              icms.parse! doc.css(icms_type.root_name).to_xml

              object_to_attribute icms
            end
          end
        end

        def object_to_attribute(object)
          self.class.attr_icms.each do |attribute|
            if object.respond_to? "#{attribute}"
              send("#{attribute}=", object.send(attribute))
            end
          end
        end

        def reset_values
          self.class.attr_icms.each { |attribute| send("#{attribute}=", nil) }
        end
      end
    end
  end
end
