require_relative "ICMSNormalBase"

module Nfe
  module Models
    module Taxes
      class ICMS70 < ICMSNormalBase

        attr_xml :modBC, type: :integer, required: true
        attr_xml :pRedBC, type: :float, required: true
        attr_xml :vBC, type: :float, required: true
        attr_xml :pICMS, type: :float, required: true
        attr_xml :vICMS, type: :float, required: true
        attr_xml :modBCST, type: :integer, required: true
        attr_xml :pMVAST, type: :float
        attr_xml :pRedBCST, type: :float
        attr_xml :vBCST, type: :float, required: true
        attr_xml :pICMSST, type: :float, required: true
        attr_xml :vICMSST, type: :float, required: true

        validates :modBC,
          presence: true,
          numericality: true,
          inclusion: { in: [0, 1, 2, 3] }

        validates :pRedBC,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 999.99,
          }

        validates :vBC,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99
          }

        validates :pICMS,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 999.99
          }

        validates :vICMS,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99
          }

          validates :modBCST,
            presence: true,
            numericality: true,
            inclusion: { in: [0, 1, 2, 3, 4, 5] }

        validates :pMVAST,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 999.99,
            allow_blank: true
          }

        validates :pRedBCST,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 999.99,
            allow_blank: true
          }

        validates :vBCST,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99
          }

        validates :pICMSST,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 999.99
          }

        validates :vICMSST,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99
          }

      end
    end
  end
end
