module Nfe
  module Models
    module Taxes
      class ICMSST
        include ActiveModel::Model
        include Helpers::XmlModel::NfeModel

        attr_xml :orig, type: :integer, required: :true
        attr_xml :cst, xml_alias: "CST"
        attr_xml :vBCSTRet, type: :float, required: :true
        attr_xml :vICMSSTRet, type: :float, required: :true
        attr_xml :vBCSTDest, type: :float, required: :true
        attr_xml :vICMSSTDest, type: :float, required: :true

        validates :orig,
          presence: true,
          numericality: true,
          inclusion: { in: [0, 1, 2] }

        validates :vBCSTRet,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99
          }

        validates :vICMSSTRet,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99
          }

        validates :vBCSTDest,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99
          }

        validates :vICMSSTDest,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99
          }

        def cst
          "41"
        end
      end
    end
  end
end
