module Nfe
  module Models
    module Taxes
      class IPITrib
        include ActiveModel::Model
        include Helpers::XmlModel::NfeModel

        attr_xml :cst, required: :true, xml_alias: "CST"
        attr_xml :vBC, type: :float
        attr_xml :pIPI, type: :float
        attr_xml :qUnid, type: :float, format: "%0.4f"
        attr_xml :vUnid, type: :float, format: "%0.4f"
        attr_xml :vIPI, type: :float, required: :true 

        validates :cst,
          presence: true,
          inclusion: { in: ["00", "49", "50", "99"] }

        validates :vBC,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99
          }

        validates :pIPI,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 999.99
          }

        validates :qUnid,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 999_999_999_999.9999
          }

        validates :vUnid,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 99_999_999_999.9999
          }

        validates :vIPI,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99
          }
      end
    end
  end
end
