module Nfe
  module Models
    module Taxes
      class ISSQN
        include ActiveModel::Model
        include Helpers::XmlModel::NfeModel

        attr_xml :vBC, type: :float, required: true
        attr_xml :vAliq, type: :float, required: true
        attr_xml :vISSQN, type: :float, required: true
        attr_xml :cMunFG, required: true
        attr_xml :cListServ, required: true
        attr_xml :cSitTrib, type: :enumerize, required: true

        extend Enumerize

        enumerize :cSitTrib, in: { normal: "N", retida: "R",
          substituta: "S", isenta: "I" }

        validates :vBC,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99
          }

        validates :vAliq,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 999.99
          }

        validates :vISSQN,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99
          }

        validates :cMunFG,
          presence: true,
          length: { is: 7 },
          numericality: { only_integer: true }

        validates :cListServ,
          presence: true,
          numericality: { only_integer: true,
            less_than_or_equal_to: 9999
          }

        validates :cSitTrib,
          presence: true
      end
    end
  end
end
