module Nfe
  module Models
    module Taxes
      class PISCOFINSAliq
        include ActiveModel::Model

        attr_accessor :cst, :vBC, :tax, :value

        def self.csts
          ["01", "02"]
        end

        validates :cst,
          presence: true,
          inclusion: { in: PISCOFINSAliq.csts }

        validates :vBC,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99
          }

        validates :tax,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 999.99
          }

        validates :value,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99
          }

        def initialize(xml_tax, xml_value)
          @xml_tax = xml_tax
          @xml_value = xml_value
        end

        def parse!(xml)
          doc = Nokogiri::XML(xml)

          @cst = doc.css("CST").text.to_s
          @vBC = doc.css("vBC").text.to_f
          @tax = doc.css(@xml_tax).text.to_f
          @value = doc.css(@xml_value).text.to_f
        end

        def to_xml
          xml = Builder::XmlMarkup.new

          xml.CST "%2.2d" % @cst
          xml.vBC "%0.2f" % @vBC
          xml.tag!(@xml_tax) { |node| node << "%0.2f" % @tax }
          xml.tag!(@xml_value) { |node| node << "%0.2f" % @value }

          xml.target!
        end
      end
    end
  end
end
