module Nfe
  module Models
    module Taxes
      class PISCOFINSOutr
        include ActiveModel::Model
        include Helpers::XmlModel::NfeModel

        attr_xml :cst, required: true, xml_alias: "CST"
        attr_xml :vBC, type: :float
        attr_xml :qBCProd, type: :float, format: "%0.4f"
        attr_xml :vAliqProd, type: :float, format: "%0.4f"

        attr_accessor :tax, :value

        def self.csts
          ["49", "50", "51", "52", "53", "54", "55", "56", "60",
            "61", "62", "63", "64", "65", "66", "67", "70", "71", "72", "73", "74",
            "75", "98", "99"]
        end

        validates :cst,
          presence: true,
          inclusion: { in: PISCOFINSOutr.csts
          }

        validates :vBC,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99,
            allow_blank: true
          }

        validates :tax,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 999.99,
            allow_blank: true
          }

        validates :qBCProd,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 999_999_999_999.9999,
            allow_blank: true
          }

        validates :vAliqProd,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 99_999_999_999.9999,
            allow_blank: true
          }

        validates :value,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99
          }

        def initialize(xml_tax, xml_value)
          @xml_tax = xml_tax
          @xml_value = xml_value
        end


        def parse!(xml)
          doc = Nokogiri::XML(xml)

          parse_elements doc
          @tax = doc.css(@xml_tax).present? ? doc.css(@xml_tax).text.to_f : nil
          @value = doc.css(@xml_value).text.to_f
        end

        def to_xml
          xml = Builder::XmlMarkup.new

          add_elements xml, :cst, :vBC
          xml.tag!(@xml_tax) { |node| node << "%0.2f" % @tax } if @tax.present?
          add_elements xml, :qBCProd, :vAliqProd
          xml.tag!(@xml_value) { |node| node << "%0.2f" % @value }

          xml.target!
        end
      end
    end
  end
end
