module Nfe
  module Models
    module Taxes
      class PISCOFINSQtde
        include ActiveModel::Model

        attr_accessor :qBCProd, :vAliqProd, :value

        def cst
          "03"
        end

        validates :qBCProd,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 999_999_999_999.9999
          }

        validates :vAliqProd,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 99_999_999_999.9999
          }

        validates :value,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99
          }

        def initialize(xml_value)
          @xml_value = xml_value
        end

        def parse!(xml)
          doc = Nokogiri::XML(xml)

          @qBCProd = doc.css("qBCProd").text.to_f
          @vAliqProd = doc.css("vAliqProd").text.to_f
          @value = doc.css(@xml_value).text.to_f
        end

        def to_xml
          xml = Builder::XmlMarkup.new

          xml.CST self.cst
          xml.qBCProd "%0.4f" % @qBCProd
          xml.vAliqProd "%0.4f" % @vAliqProd
          xml.tag!(@xml_value) { |node| node << "%0.2f" % @value }

          xml.target!
        end
      end
    end
  end
end
