module Nfe
  module Models
    module Taxes
      class PISCOFINSST
        include ActiveModel::Model

        attr_accessor :vBC, :tax, :qBCProd, :vAliqProd, :value

        validates :vBC,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99,
            allow_blank: true
          }

        validates :tax,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 999.99,
            allow_blank: true
          }

        validates :qBCProd,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 999_999_999_999.9999,
            allow_blank: true
          }

        validates :vAliqProd,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 99_999_999_999.9999,
            allow_blank: true
          }

        validates :value,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99,
            allow_blank: true
          }

        def initialize(xml_tax, xml_value)
          @xml_tax = xml_tax
          @xml_value = xml_value
        end

        def present?
          @vBC.present? || @tax.present? || @qBCProd.present? ||
            @vAliqProd.present? || @value.present?
        end

        def parse!(xml)
          doc = Nokogiri::XML(xml)

          @vBC = doc.css("vBC").present? ? doc.css("vBC").text.to_f : nil
          @tax = doc.css(@xml_tax).present? ? doc.css(@xml_tax).text.to_f : nil
          @qBCProd = doc.css("qBCProd").present? ? doc.css("qBCProd").text.to_f : nil
          @vAliqProd = doc.css("vAliqProd").present? ? doc.css("vAliqProd").text.to_f : nil
          @value = doc.css(@xml_value).text.to_f
        end

        def to_xml
          xml = Builder::XmlMarkup.new

          xml.vBC "%0.2f" % @vBC if @vBC.present?
          xml.tag!(@xml_tax) { |node| node << "%0.2f" % @tax } if @tax.present?
          xml.qBCProd "%0.4f" % @qBCProd if @qBCProd.present?
          xml.vAliqProd "%0.4f" % @vAliqProd if @vAliqProd.present?
          xml.tag!(@xml_value) { |node| node << "%0.2f" % @value }

          xml.target!
        end
      end
    end
  end
end
