module Nfe
  module Models
    module Taxes
      class Imposto
        include ActiveModel::Model
        include Helpers::XmlModel::NfeModel

        attr_xml :vTotTrib, type: :float
        attr_xml :icms, type: :node, xml_alias: "ICMS"
        attr_xml :ipi, type: :node, xml_alias: "IPI"
        attr_xml :ii, type: :node, xml_alias: "II"
        attr_xml :issqn, type: :node, xml_alias: "ISSQN"
        attr_xml :pis, type: :node, xml_alias: "PIS"
        attr_xml :pisst, type: :node, xml_alias: "PISST"
        attr_xml :cofins, type: :node, xml_alias: "COFINS"
        attr_xml :cofinsst, type: :node, xml_alias: "COFINSST"

        validates :vTotTrib,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9_999_999_999_999.99,
            allow_blank: true
          }

        validates :icms,
          presence: true

        validate { validate_optional_attributes :icms, :ipi, :ii, :pis, :cofins,
          :issqn, :pisst, :cofinsst }

        def initialize
          @icms = ICMS.new
          @ipi = IPI.new
          @ii = II.new
          @pis = PISCOFINS.new("pis")
          @pisst = PISCOFINSST.new("pPIS","vPIS")
          @cofins = PISCOFINS.new("cofins")
          @cofinsst = PISCOFINSST.new("pCOFINS","vCOFINS")
          @issqn = ISSQN.new
        end
      end
    end
  end
end
