module Nfe
  module Models
    module Transport
      class Transp
        include ActiveModel::Model
        include Helpers::XmlModel::NfeModel

        attr_xml :modFrete, type: :enumerize, required: true
        attr_xml :transporta, type: :node
        attr_xml :retTransp, type: :node
        attr_xml :veicTransp, type: :node
        attr_xml :reboque, type: :collection,
          klass: "Nfe::Models::Transport::VeicTranspReboque"
        attr_xml :vagao, type: :text
        attr_xml :balsa, type: :text
        attr_xml :vol, type: :collection, klass: "Nfe::Models::Transport::Vol"

        extend Enumerize

        enumerize :modFrete, in: { emitter: 0, recipient: 1, third_party: 2,
          no_freight: 9}

        validates :modFrete,
          presence: true

        validates :vagao,
          length: { maximum: 20 }

        validates :balsa,
          length: { maximum: 20 }

        validates :reboque,
           length: { maximum: 5, message: :less_than_or_equal_to }

        validate do
          validate_optional_attributes :transporta, :retTransp, :veicTransp,
            :reboque, :vol
        end

        def initialize
          @transporta = Transporta.new
          @retTransp = RetTransp.new
          @veicTransp = VeicTranspReboque.new
          @reboque = []
          @vol = []
        end
      end
    end
  end
end
