module Nfe
  module Models
    module Transport
      class Vol
        include ActiveModel::Model
        include Helpers::XmlModel::NfeModel

        attr_xml :qVol, type: :integer
        attr_xml :esp
        attr_xml :marca
        attr_xml :nVol
        attr_xml :pesoL, type: :float, format: "%0.3f"
        attr_xml :pesoB, type: :float, format: "%0.3f"
        attr_xml :lacres, type: :collection,
          klass: "Nfe::Models::Transport::Lacres"

        validates :qVol,
          length: { in: 1..15, allow_blank: true },
          numericality: { only_integer: true, allow_blank: true }

        validates :esp,
          length: { in: 1..60, allow_blank: true }

        validates :marca,
          length: { in: 1..60, allow_blank: true }

        validates :nVol,
          length: { in: 1..60, allow_blank: true }

        validates :pesoL,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9999999999999.99
          }

        validates :pesoB,
          numericality: {
            greater_than_or_equal_to: 0,
            less_than_or_equal_to: 9999999999999.99
          }

        validate { validate_optional_attributes :lacres }

        def initialize
          @lacres = []
        end
      end
    end
  end
end
