require_relative "base"

module Nfe
  module Operations
    class Event < Base
      def initialize(certificate, private_key, environment, nfe_key,
        sequential, date_time, detEvent, use_schema: true)
        super(certificate, private_key, use_schema: use_schema)

        @retEnvEvento = Services::Regress::RetEnvEvento.new
        @environment = environment
        @detEvent = detEvent
        @key = Helpers::NfeKey.parse(nfe_key)
        @date_time = date_time

        @object = object_to_send(nfe_key, sequential)

      rescue NfeKeyError
        errors.add(:base, I18n.t("xml.invalid_nfe_key"))
      end

      def processed?
        @retEnvEvento.cStat.to_i == processed_lot_code
      end

      def success?
        success_codes.include? @retEnvEvento.retEvento.cStat
      end

      def rejected?
        @retEnvEvento.retEvento.cStat.to_i > 200
      end

      def message
        @retEnvEvento.retEvento.xMotivo if processed?
      end

      def event_xml
        if success?
          procEventoNFe = Nfe::Models::ProcEventoNFe.new(@object.evento, @retEnvEvento.retEvento)
          procEventoNFe.to_xml
        end
      end

      def protocol_number
        @retEnvEvento.retEvento.nProt if success?
      end

      protected
      def before_execute
        super

        sign @object.evento, @object.evento.id
      end

      def handle_result(xml)
        @retEnvEvento.parse! xml

        errors.add(:base, @retEnvEvento.xMotivo) if !processed?

        @retEnvEvento
      end

      def xpath
        "//ns:retEnvEvento"
      end

      def schema_name
        ws_info.schema
      end

      def ws_info
        Webservices::Wsinfo::Event.new(@environment, @key.uf, @detEvent.kind)
      end

      private
      def object_to_send(nfe_key, sequential)
        envEvento = Services::Dispatch::EnvEvento.new

        envEvento.versao = ws_info.version
        envEvento.idLote = default_lot
        @detEvent.versao = ws_info.version
        envEvento.evento = event(nfe_key, sequential)

        envEvento
      end

      def default_lot
        1
      end

      def event(nfe_key, sequential)
        event = Nfe::Models::Evento.new

        event.versao = ws_info.version
        event.uf = @key.uf
        event.tpAmb = @environment
        event.tpEvento = @detEvent.tpEvent
        event.nSeqEvento = sequential
        event.cnpj = @key.cnpj
        event.chNFe = nfe_key
        event.dhEvento = @date_time
        event.detEvento = @detEvent

        event
      end

      def processed_lot_code
        128
      end

      def success_codes
        [135, 136]
      end
    end
  end
end
