require_relative "event"

module Nfe
  module Operations
    class NfeCancel < Event
      def initialize(certificate, private_key, environment:, nfe_key:,
        date_time:, protocol:, reason:, use_schema: true)

        @event = object_event(protocol, reason)

        super(certificate, private_key, environment, nfe_key,
          sequential_number, date_time, @event, use_schema: use_schema)
      end

      def canceled_xml
        event_xml
      end

      protected
      def object_event(protocol, reason)
        event = Services::Dispatch::DetEventoCancel.new

        event.nProt = protocol
        event.xJust = reason

        event
      end

      def valid?
        if @event.nProt.to_s.length < 15
          errors.add(:base, I18n.t("nfe_cancel.invalid_protocol"))
        end

        if @event.xJust.to_s.length < 15
          errors.add(:base, I18n.t("nfe_cancel.invalid_reason"))
        end

        super
      end

      private
      def sequential_number
        1
      end
    end
  end
end
