require_relative "base"

module Nfe
  module Operations
    class NfeDiscard < Base
      def initialize(certificate:, private_key:, environment:, uf:, cnpj:, series:,
        reason:, initial_number:, final_number: nil, use_schema: true)
        super(certificate, private_key, use_schema: use_schema)

        @uf = uf
        @environment = environment

        @object = object_to_send(cnpj, series, reason, initial_number,
          final_number)

        @retInutNFe = Services::Regress::RetInutNFe.new
      end

      def success?
        @retInutNFe.cStat == discard_code
      end

      def rejected?
        @retInutNFe.cStat.to_i > 200
      end

      def message
        @retInutNFe.xMotivo if rejected? || success?
      end

      def discard_xml
        if success?
          procInutNFe = Nfe::Models::ProcInutNFe.new(@object, @retInutNFe)
          procInutNFe.to_xml
        end
      end

      protected
      def before_execute
        sign @object, @object.id
      end

      def handle_result(xml)
        @retInutNFe.parse! xml
        @retInutNFe
      end

      def valid?
        if @object.uf.to_s.length != 2
          errors.add(:base, I18n.t("nfe_discard.invalid_uf"))
        end

        if @object.cnpj.to_s.length != 14
          errors.add(:base, I18n.t("nfe_discard.invalid_cnpj"))
        end

        if @object.serie.to_i <= 0
          errors.add(:base, I18n.t("nfe_discard.invalid_series"))
        end

        if @object.nNFIni.to_i <= 0
          errors.add(:base, I18n.t("nfe_discard.invalid_number"))
        end

        if @object.xJust.to_s.length < 15
          errors.add(:base, I18n.t("nfe_discard.invalid_reason"))
        end

        super
      end

      def xpath
        "//ns:retInutNFe"
      end

      def schema_name
        ws_info.schema
      end

      def ws_info
        Webservices::Wsinfo::NfeDisenable.new(@environment, @uf)
      end

      private
      def object_to_send(cnpj, series, reason, initial_number,
        final_number = nil)
        inutNFe = Services::Dispatch::InutNFe.new

        inutNFe.tpAmb = @environment
        inutNFe.versao = ws_info.version
        inutNFe.uf = @uf
        inutNFe.ano = Date.today.strftime("%y")
        inutNFe.cnpj = cnpj
        inutNFe.serie = series
        inutNFe.nNFIni = initial_number
        inutNFe.nNFFin = final_number || initial_number
        inutNFe.xJust = reason
        inutNFe
      end

      def discard_code
        102
      end
    end
  end
end
