require_relative "base"

module Nfe
  module Operations
    class NfeSend < Base
      include ActiveModel::Model

      def initialize(certificate, private_key, nfe, lot=nil, use_schema: true)
        super(certificate, private_key, use_schema: use_schema)
        @nfe = nfe
        @lot = lot || lot_default

        @retEnviNFe = Services::Regress::RetEnviNFe.new
      end

      def authorized?
        @retEnviNFe.protNFe.cStat == authorized_code
      end

      def denied?
        denied_codes.include? @retEnviNFe.protNFe.cStat
      end

      def rejected?
        rejected_code? && !denied?
      end

      def authorized_xml
        if authorized?
          nfeProc = Nfe::Models::NFeProc.new(@nfe, @retEnviNFe.protNFe)
          nfeProc.to_xml
        end
      end

      def protocol_number
        if authorized?
          @retEnviNFe.protNFe.nProt
        end
      end

      def nfe_status
        if authorized?
          :authorized
        elsif denied?
          :denied
        elsif rejected?
          :rejected
        else
          :pending
        end
      end

      def nfe_key
        if authorized?
          @retEnviNFe.protNFe.chNFe
        end
      end

      def processed?
        authorized? || rejected? || denied?
      end

      def message
        @retEnviNFe.protNFe.xMotivo if processed?
      end

      protected
      def before_execute
        sign @nfe, @nfe.infNFe.id
        @object = Services::Dispatch::EnviNFe.new(@nfe, @lot, ws_info.version)
      end

      def ws_info
        Webservices::Wsinfo::NfeSend.new(environment, uf)
      end

      def handle_result(xml)
        @retEnviNFe.parse! xml

        if !processed_lot_code.include? @retEnviNFe.cStat
          errors.add(:base, @retEnviNFe.xMotivo)
        end

        @retEnviNFe
      end

      def xpath
        "//ns:retEnviNFe"
      end

      def schema_name
        ws_info.schema
      end

      private
      def authorized_code
        100
      end

      def processed_lot_code
        [104, 110]
      end

      def denied_codes
        [301, 302]
      end

      def rejected_code?
        @retEnviNFe.protNFe.cStat.to_i > 200
      end

      def uf
        Nfe::Helpers::UfHelper.acronym(@nfe.infNFe.ide.cUF)
      end

      def environment
        @nfe.infNFe.ide.tpAmb
      end

      def lot_default
        "#{@nfe.infNFe.ide.serie}#{@nfe.infNFe.ide.nNF}".to_i
      end
    end
  end
end
