module Nfe
  module Services
    module Dispatch
      class ConsCad
        include ActiveModel::Model
        include Helpers::XmlModel::NfeModel

        attr_xml :uf, required: true, xml_alias: "UF"
        attr_xml :ie, xml_alias: "IE"
        attr_xml :cnpj, xml_alias: "CNPJ"
        attr_xml :cpf, xml_alias: "CPF"

        attr_accessor :versao, :infConsEntrada

        extend Enumerize

        enumerize :tpAmb, in: { production: 1, test: 2 }

        validates :versao,
          presence: true,
          length: { is: 4 }

        validates :uf,
          presence: true,
          length: { is: 2 }

        validates :cnpj,
          presence: { if: -> { cpf.blank? && ie.blank?} },
          cnpj: true,
          length: { is: 14, allow_blank: true }

        validates :cpf,
          presence: { if: -> { cnpj.blank? && ie.blank? } },
          cpf: true,
          length: { is: 11, allow_blank: true }

        validates :ie,
          presence: { if: -> { cnpj.blank? && cpf.blank? } },
          ie: { uf: "uf" },
          length: { in: 2..14, allow_blank: true }

        def to_xml
          xml = Builder::XmlMarkup.new

          xml.ConsCad(xmlns: Nfe::Models::NFe.namespace, versao: versao ) do |env|
            xml.infCons do |node|
              node.xServ "CONS-CAD"
              add_elements node
            end
          end

          xml.target!
        end
      end
    end
  end
end
