module Nfe
  module Services
    module Dispatch
      class InutNFe
        include ActiveModel::Model
        include Models::Concerns::Ufable
        include Helpers::XmlModel::NfeModel

        attr_xml :tpAmb, type: :enumerize, required: true
        attr_xml :xServ
        attr_xml :cUF, type: :integer, required: true
        attr_xml :ano, required: true
        attr_xml :cnpj, required: true, xml_alias: "CNPJ"
        attr_xml :mod
        attr_xml :serie, required: true
        attr_xml :nNFIni, required: true
        attr_xml :nNFFin, required: true
        attr_xml :xJust, required: true

        attr_accessor :versao, :signature

        extend Enumerize

        enumerize :tpAmb, in: { production: 1, test: 2 }

        validates :versao,
          length: { is: 4 }

        validates :cUF,
          length: { is: 2 }

        validates :ano,
          length: { is: 2 }

        validates :cnpj,
          length: { is: 14 }

        validates :serie,
          length: { in: 1..3 }

        validates :nNFIni,
          length: { in: 1..9 }

        validates :nNFFin,
          length: { in: 1..9 }

        validates :xJust,
          length: { in: 15..255 }

        def id
          "ID#{cUF}#{ano}#{@cnpj}#{mod}#{serie_f}#{nNFIni_f}#{nNFFin_f}"
        end

        def to_xml
          xml = Builder::XmlMarkup.new

          xml.inutNFe({xmlns: Models::NFe.namespace, versao: versao}) do |nfe|

            nfe.infInut(Id: id) { |node| add_elements node }

            nfe.Signature(xmlns: Models::Signature.namespace) do |node|
              node << @signature.value
            end

          end

          xml.target!
        end

        private
        def mod
          55
        end

        def xServ
          "INUTILIZAR"
        end

        def serie_f
          "%03d" % serie.to_i
        end

        def nNFIni_f
          "%09d" % nNFIni.to_i
        end

        def nNFFin_f
          "%09d" % nNFFin.to_i
        end
      end
    end
  end
end
