module Nfe
  module Webservices
    class Sender
      extend ActiveModel::Naming

      attr_reader :result, :errors

      def initialize(certificate, private_key, ws_info)
        @certificate = certificate
        @private_key = private_key
        @errors = ActiveModel::Errors.new(self)
        @ws_info = ws_info
      end

      def execute(object)
        call_client_soap(object) if message_size_valid?(object.to_xml)
        @errors.empty?
      end

      private
      def call_client_soap(object)
        @result = client_soap.call(object.to_xml)
      rescue Exception => error
        @errors.add(:base, error.to_s)
      end

      def client_soap
        ClientSoap.new(@certificate, @private_key, @ws_info)
      end

      def message_size_valid?(xml)
        quinhentos_kilobytes = 512000

        if xml.bytesize > quinhentos_kilobytes
          @errors.add(:base, I18n.t("webservice.dispatch.exceeded_size"))
        end

        @errors.empty?
      end
    end
  end
end
