module Xmldsig
  class Reference
    attr_accessor :reference, :errors

    class ReferencedNodeNotFound < Exception; end

    def initialize(reference)
      @reference = reference
      @errors = []
    end

    def sign
      self.digest_value = calculate_digest_value
    end

    def validate_digest_value
      unless digest_value == calculate_digest_value
        @errors << :digest_value
      end
    end

    def referenced_node
      if reference_uri && reference_uri != ""
        id = reference_uri[1..-1]
        if ref = document.dup.at_xpath("//*[@Id|@ID='#{id}' or @wsu:Id='#{id}']", Xmldsig::NAMESPACES)
          ref
        else
          raise(ReferencedNodeNotFound, "Could not find the referenced node #{id}")
        end
      else
        document.dup.root
      end
    end

    def reference_uri
      reference.get_attribute("URI")
    end

    def digest_value
      Base64.decode64 reference.at_xpath("descendant::ds:DigestValue", Xmldsig::NAMESPACES).content
    end

    def document
      reference.document
    end

    private
    def calculate_digest_value
      node = transforms.apply(referenced_node)
      digest_method.digest node
    end

    def digest_method
      algorithm = reference.at_xpath("descendant::ds:DigestMethod", Xmldsig::NAMESPACES).get_attribute("Algorithm")
      case algorithm
        when "http://www.w3.org/2001/04/xmlenc#sha256"
          Digest::SHA2
        when "http://www.w3.org/2000/09/xmldsig#sha1"
          Digest::SHA1
      end
    end

    def digest_value=(digest_value)
      reference.at_xpath("descendant::ds:DigestValue", Xmldsig::NAMESPACES).content =
        Base64.encode64(digest_value).chomp
    end

    def transforms
      Transforms.new(reference.xpath("descendant::ds:Transform", Xmldsig::NAMESPACES))
    end
  end
end
