module Xmldsig
  class SignedDocument
    attr_accessor :document

    def initialize(document)
      @document = Nokogiri::XML::Document.parse(document)
    end

    def sign(private_key = nil, certificate = nil, &block)
      signatures.each { |signature| signature.sign(private_key, certificate, &block) }
      document.root.to_s
    end

    def validate(certificate = nil, &block)
      signatures.any? && signatures.all? { |signature| signature.valid?(certificate, &block) }
    end

    def signatures
      document.xpath("//ds:Signature", Xmldsig::NAMESPACES).reverse.collect do |node|
        Signature.new(node)
      end || []
    end

    def signed_nodes
      signatures.flat_map(&:references).map(&:referenced_node)
    end
  end
end
